use crate::{Map, Val};

use serde::{
    self,
    de::{
        value::{MapDeserializer, SeqDeserializer},
        Visitor,
    },
};

pub type ValDeserializerError = serde::de::value::Error;

mod enum_de;
mod val;
mod val_e;
mod val_ref;

pub(crate) fn visit_array<'de, V>(
    array: Vec<Val>,
    visitor: V,
) -> Result<V::Value, ValDeserializerError>
where
    V: Visitor<'de>,
{
    let mut deserializer = SeqDeserializer::new(array.into_iter());
    let seq = visitor.visit_seq(&mut deserializer)?;
    deserializer.end().map(|_ok| seq)
}

pub(crate) fn visit_object<'de, V>(
    object: Map,
    visitor: V,
) -> Result<V::Value, ValDeserializerError>
where
    V: Visitor<'de>,
{
    let object = object.into_entries().0;
    let mut deserializer = MapDeserializer::new(object.into_iter());
    let map = visitor.visit_map(&mut deserializer)?;
    // make sure no elements remain
    deserializer.end().map(|_ok| map)
}
