use serde::{
    de::{IntoDeserializer, Visitor},
    forward_to_deserialize_any, Deserializer,
};

use crate::{Key, Val, ValE};

use super::{val_e::deserialize_val_e_enum, val_ref::deserialize_val_ref_enum};

impl<'de> Deserializer<'de> for Val {
    type Error = serde::de::value::Error;

    #[inline]
    fn deserialize_any<V>(self, visitor: V) -> Result<V::Value, Self::Error>
    where
        V: serde::de::Visitor<'de>,
    {
        ValE::from(self).deserialize_any(visitor)
    }

    forward_to_deserialize_any! {
        bool i8 i16 i32 i64 i128 u8 u16 u32 u64 u128 f32 f64 char str string
        bytes byte_buf unit unit_struct newtype_struct seq tuple
        tuple_struct map struct identifier ignored_any
    }

    #[inline]
    fn deserialize_option<V>(self, visitor: V) -> Result<V::Value, Self::Error>
    where
        V: Visitor<'de>,
    {
        ValE::from(self).deserialize_option(visitor)
    }

    #[inline]
    fn deserialize_enum<V>(
        self,
        _name: &str,
        _variants: &'static [&'static str],
        visitor: V,
    ) -> Result<V::Value, Self::Error>
    where
        V: Visitor<'de>,
    {
        deserialize_val_e_enum(ValE::from(self), visitor)
    }
}

impl<'de> IntoDeserializer<'de> for Val {
    type Deserializer = Val;

    fn into_deserializer(self) -> Self::Deserializer {
        self
    }
}

impl<'de> Deserializer<'de> for &'de Val {
    type Error = serde::de::value::Error;

    #[inline]
    fn deserialize_any<V>(self, visitor: V) -> Result<V::Value, Self::Error>
    where
        V: serde::de::Visitor<'de>,
    {
        self.direct_ref().deserialize_any(visitor)
    }

    forward_to_deserialize_any! {
        bool i8 i16 i32 i64 i128 u8 u16 u32 u64 u128 f32 f64 char str string
        bytes byte_buf unit unit_struct newtype_struct seq tuple
        tuple_struct map struct identifier ignored_any
    }

    #[inline]
    fn deserialize_option<V>(self, visitor: V) -> Result<V::Value, Self::Error>
    where
        V: Visitor<'de>,
    {
        self.direct_ref().deserialize_option(visitor)
    }

    #[inline]
    fn deserialize_enum<V>(
        self,
        _name: &'de str,
        _variants: &'static [&'static str],
        visitor: V,
    ) -> Result<V::Value, Self::Error>
    where
        V: Visitor<'de>,
    {
        deserialize_val_ref_enum(self.direct_ref(), visitor)
    }
}

impl<'de> IntoDeserializer<'de> for &'de Val {
    type Deserializer = &'de Val;

    fn into_deserializer(self) -> Self::Deserializer {
        self
    }
}

impl<'de> IntoDeserializer<'de> for Key {
    type Deserializer = Val;

    fn into_deserializer(self) -> Self::Deserializer {
        self.into()
    }
}

// TODO: Move this somewhere better
#[cfg(test)]
mod test {
    use crate::{from_val, Val};

    #[test]
    fn option_deserializes() {
        let val: Option<String> = from_val(Val::str("test")).unwrap();
        assert_eq!(val, Some("test".to_owned()));
    }
}