use serde::Serialize;

use super::super::{Val, ValE};

impl Serialize for ValE {
    #[inline]
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: serde::Serializer,
    {
        match self {
            ValE::Null => serializer.serialize_none(),
            ValE::Bool(b) => serializer.serialize_bool(*b),
            ValE::Number(n) => serializer.serialize_f64(n.into_inner()),
            ValE::ShortString(s) => serializer.serialize_str(s.as_ref()),
            ValE::LongString(s) => serializer.serialize_str(s.as_ref()),
            ValE::Array(a) => serializer.collect_seq(a.as_ref().iter()),
            ValE::Object(o) => {
                serializer.collect_map(o.entries().iter().map(|(k, v)| (k.as_ref(), v)))
            }
        }
    }
}

impl Serialize for Val {
    #[inline]
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: serde::Serializer,
    {
        let unsafe_clone = Into::<ValE>::into(Val(self.0));
        let result = unsafe_clone.serialize(serializer)?;
        std::mem::forget(unsafe_clone);
        Ok(result)
    }
}
